@@ -5,17 +5,14 @@ from pathlib import Path
 from PIL import Image
 import time
 import progressbar
+from .util.image_handling import convert_graymap_image_to_hsvmap_image, rescale_img_to_simulation_coordinates
+from .util.constants import *
+
 
 import numpy as np
-from .backend_functions import backend as bd
+from .util.backend_functions import backend as bd
 
 
-m = 1.
-cm = 1e-2
-mm = 1e-3
-um = 1e-6
-nm = 1e-9
-W = 1
 
 
 
@@ -34,13 +31,16 @@ class MonochromaticField:
         intensity: intensity of the field
         """
         global bd
-        from .backend_functions import backend as bd
+        from .util.backend_functions import backend as bd
 
         self.extent_x = extent_x
         self.extent_y = extent_y
 
-        self.x = self.extent_x*(bd.arange(Nx)-Nx//2)/Nx
-        self.y = self.extent_y*(bd.arange(Ny)-Ny//2)/Ny
+        self.dx = self.extent_x/Nx
+        self.dy = self.extent_y/Ny
+
+        self.x = self.dx*(bd.arange(Nx)-Nx//2)
+        self.y = self.dy*(bd.arange(Ny)-Ny//2)
         self.xx, self.yy = bd.meshgrid(self.x, self.y)
 
         self.Nx = bd.int(Nx)
@@ -54,14 +54,10 @@ class MonochromaticField:
         """
         Creates a slit centered at the point (x0, y0) with width width and height height
         """
-        t = bd.select(
-            [
-                ((self.xx > (x0 - width / 2)) & (self.xx < (x0 + width / 2)))
-                & ((self.yy > (y0 - height / 2)) & (self.yy < (y0 + height / 2))),
-                bd.ones_like(self.E, dtype=bool),
-            ],
-            [bd.ones_like(self.E), bd.zeros_like(self.E)],
-        )
+        t = bd.where((((self.xx > (x0 - width / 2)) & (self.xx < (x0 + width / 2)))
+                        & ((self.yy > (y0 - height / 2)) & (self.yy < (y0 + height / 2)))),
+                        bd.ones_like(self.E), bd.zeros_like(self.E))
+
         self.E = self.E*t
 
         self.I = bd.real(self.E * bd.conjugate(self.E))  
@@ -91,6 +87,7 @@ class MonochromaticField:
 
 
 
+
     def add_diffraction_grid(self, D, a, Nx, Ny):
         """
         Creates a diffraction_grid with Nx *  Ny slits with separation distance D and width a
@@ -132,7 +129,7 @@ class MonochromaticField:
         return t
 
 
-    def add_aperture_from_image(self, amplitude_mask_path, image_size = None):
+    def add_aperture_from_image(self, amplitude_mask_path= None, phase_mask_path= None, image_size = None, phase_mask_format = 'hsv'):
         """
         Load the image specified at "amplitude_mask_path" as a numpy graymap array represeting the amplitude transmittance of the aperture. 
         The image is centered on the plane and its physical size is specified in image_size parameter as image_size = (float, float)
@@ -140,41 +137,43 @@ class MonochromaticField:
         - If image_size isn't specified, the image fills the entire aperture plane
         """
 
+        if amplitude_mask_path != None:
+            
+            #load the amplitude_mask image
+            img = Image.open(Path(amplitude_mask_path))
+            img = img.convert("RGB")
 
-        img = Image.open(Path(amplitude_mask_path))
-        img = img.convert("RGB")
+            rescaled_img = rescale_img_to_simulation_coordinates(img, image_size , self.extent_x,self.extent_y, self.Nx, self.Ny)
+            imgRGB = np.asarray(rescaled_img) / 255.0
 
-        img_pixels_width, img_pixels_height = img.size
+            t = 0.2990 * imgRGB[:, :, 0] + 0.5870 * imgRGB[:, :, 1] + 0.1140 * imgRGB[:, :, 2]
+            t = bd.array(np.flip(t, axis = 0))
 
-        if image_size != None:
-            new_img_pixels_width, new_img_pixels_height = int(np.round(image_size[0] / self.extent_x  * self.Nx)),  int(np.round(image_size[1] / self.extent_y  * self.Ny))
-        else:
-            #by default, the image fills the entire aperture plane
-            new_img_pixels_width, new_img_pixels_height = self.Nx, self.Ny
+            self.E = self.E*t
 
-        img = img.resize((new_img_pixels_width, new_img_pixels_height))
 
-        dst_img = Image.new("RGB", (self.Nx, self.Ny), "black" )
-        dst_img_pixels_width, dst_img_pixels_height = dst_img.size
+        if phase_mask_path != None:
+            from matplotlib.colors import rgb_to_hsv
 
-        Ox, Oy = (dst_img_pixels_width-new_img_pixels_width)//2, (dst_img_pixels_height-new_img_pixels_height)//2
-        
-        dst_img.paste( img , box = (Ox, Oy ))
+            #load the phase_mask image
+            img = Image.open(Path(phase_mask_path))
+            img = img.convert("RGB")
 
-        imgRGB = np.asarray(dst_img) / 255.0
-        imgR = imgRGB[:, :, 0]
-        imgG = imgRGB[:, :, 1]
-        imgB = imgRGB[:, :, 2]
-        t = 0.2990 * imgR + 0.5870 * imgG + 0.1140 * imgB
-        t = np.flip(t, axis = 0)
+            if phase_mask_format == 'graymap':
+                img = convert_graymap_image_to_hsvmap_image(img)
+                
+            rescaled_img = rescale_img_to_simulation_coordinates(img, image_size , self.extent_x,self.extent_y, self.Nx, self.Ny)
+            imgRGB = np.asarray(rescaled_img) / 255.0
 
-        self.E = self.E*bd.array(t)
+
+            h = rgb_to_hsv(   np.moveaxis(np.array([imgRGB[:, :, 0],imgRGB[:, :, 1],imgRGB[:, :, 2]]) , 0, -1))[:,:,0]
+            phase_mask = bd.flip(bd.array(h) * 2 * bd.pi - bd.pi, axis = 0)
+            self.E = self.E*bd.exp(1j *  phase_mask)
 
 
         # compute Field Intensity
         self.I = bd.real(self.E * bd.conjugate(self.E))  
 
-
     def add_lens(self, f, radius = None, aberration = None):
         """add a thin lens with a focal length equal to f """
 
@@ -197,8 +196,8 @@ class MonochromaticField:
         fft_c = bd.fft.fft2(self.E)
         c = bd.fft.fftshift(fft_c)
 
-        kx = 2*bd.pi*bd.fft.fftshift(bd.fft.fftfreq(self.Nx, d = self.x[1]-self.x[0]))
-        ky = 2*bd.pi*bd.fft.fftshift(bd.fft.fftfreq(self.Ny, d = self.y[1]-self.y[0]))
+        kx = 2*bd.pi*bd.fft.fftshift(bd.fft.fftfreq(self.Nx, d = self.dx))
+        ky = 2*bd.pi*bd.fft.fftshift(bd.fft.fftfreq(self.Ny, d = self.dy))
         kx, ky = bd.meshgrid(kx, ky)
 
         argument = (2 * bd.pi / self.λ) ** 2 - kx ** 2 - ky ** 2
@@ -214,6 +213,8 @@ class MonochromaticField:
         # compute Field Intensity
         self.I = bd.real(E * bd.conjugate(E))  
 
+
+
     def get_colors(self):
         """ compute RGB colors"""
 
@@ -266,7 +267,7 @@ class MonochromaticField:
     def get_longitudinal_profile(self, start_distance, end_distance, steps):
         """
         Propagates the field at n steps equally spaced between start_distance and end_distance, and returns
-        the colors and the intensity over the xz plane
+        the colors and the field over the xz plane
         """
 
         z = bd.linspace(start_distance, end_distance, steps)
